#!/usr/bin/env node
/* eslint no-console:0 no-unused-vars:0 */
const program = require('commander')
const {shell} = require('@tunnckocore/execa')
const GitUrlParse = require('git-url-parse')
const path = require('path')

program
  .option('-B, --brach <branch>', 'Release branch. Will be master by default')
  .on('--help', () => {
    console.log('  Description:')
    console.log('')
    console.log(
      ' Release a server version. Create a git tag and update a minor version in the package.json'
    )
    console.log(
      ' It is mandatory setup a $GH_TOKEN envVar to execute this command'
    )
    console.log('')
    console.log('  Examples:')
    console.log('')
    console.log('    $ sui-ssr release')
    console.log('')
  })
  .parse(process.argv)

const {branch = 'master'} = program
;(async () => {
  const cwd = process.cwd()
  const {GH_TOKEN} = process.env

  if (!GH_TOKEN) {
    throw new Error('Missing GH_TOKEN var')
  }

  const [{stdout: repoURL}] = await shell('git config --get remote.origin.url')
  const gitURL = GitUrlParse(repoURL).toString('https')
  const authURL = new URL(gitURL)
  authURL.username = GH_TOKEN
  console.log(authURL.href)

  try {
    const output = await shell(
      [
        'git config --global user.email "srv.scms.jarvis@schibsted.com"',
        'git config --global user.name "J.A.R.V.I.S"',

        `git checkout -b ${branch}`,

        'npm --no-git-tag-version version minor',

        'git add .',

        `git remote add origin-release ${authURL.href}`,

        `git commit -m "release(META): v${
          require(path.join(cwd, 'package.json')).version
        }"`,

        `git push origin-release ${branch}`
      ],
      {cwd}
    )
    console.log(output)
  } catch (err) {
    console.log(err)
  }
})()
